% Usage: octave -q ./determine_and_graph_accuracy.m <theta_file1> <theta_file2>...
% All theta files should be in the standard naming convention and
% have the same bitmask, hidden nodes and name.
% The only thing which should vary are the number of iterations.
files = argv;
if(strcmp(files{1}, "-q"))
	printf("No input files given.  Exiting...");
	quit;
end
num_of_files = length(files);

% this will hold: bitmask, hidden_nodes, iterations, accuracy
results = [];

for i=1:num_of_files; 
	printf("Processing %s\n", files{i});
	load(files{i});
	% extract the bitmask, number of hidden nodes, iterations and name from the filename
	bitmask = str2num(strsplit(files{i}, "_."){3});
	hidden_nodes = str2num(strsplit(files{i}, "_."){4});
	iter_name = strsplit(files{i}, "_."){5}; % just a temporary variable
	iter = str2num(substr(iter_name, 1, max(regexp(iter_name, "[0-9]"))));
	name = substr(iter_name, max(regexp(iter_name, "[0-9]"))+1);
	
	% Predict values for our validation set so we can see how good the algorithm actually is
	Xvalidation = csvread("validationsetx.csv");
	% y, each row is the index of the word which corresponds to the example in X
	yvalidation = csvread("validationsety.csv");
	pred = predict(Theta1, Theta2, Xvalidation);
	accuracy = mean(double(pred == yvalidation)) * 100;
	fprintf("Bitmask = %d, Hidden Nodes = %d, Iterations = %d, Name = %s, Accuracy: %f\n", bitmask, hidden_nodes, iter, name, accuracy);
	
	% Now we stash this information away in our results matrix
	results(i, 1) = bitmask;
	results(i, 2) = hidden_nodes;
	results(i, 3) = iter;
	results(i, 4) = accuracy;
end

% Make sure our results are sorted by the iteration column
results = sortrows(results, 3);

% Generate a graph using our data
plot(results(:, 3), results(:, 4), "linewidth", 5);
xlabel('Iterations');
ylabel('% Accuracy');
hold on
plot(results(:, 3), results(:, 4), "r+", 'LineWidth', 2, 'MarkerSize', 28, 'MarkerFaceColor', 'r');
legend(strcat("bitmask=", int2str(bitmask), ",hn=", int2str(hidden_nodes), ",name=", name));
hold off

% Write the graph out as a png
graph_filename = strcat("theta_values_", int2str(bitmask), "_", int2str(hidden_nodes), "_", ...
		int2str(results(1, 3)), "-", int2str(results(num_of_files, 3)), ".png");
cmd = sprintf("%s%s\n", "print -dpng ", graph_filename);
eval(cmd);








