import sys 
import splot, nnsolver, phashsolver, md5solver, humansolver

############################################################################################################################
##Defaults##################################################################################################################
############################################################################################################################

default_splot_method = splot.Splot.METHOD_FIRST_SECTION_ONLY
default_theta_files = [ "image-neural-network/theta_values.mat" ]
default_ymap_file = "image-neural-network/y_mappings.txt"
default_hashfile = "hashes"
default_number_to_find = 5
default_md5file = "md5sums.small"
default_wordmap_file = "md5sums.map"

############################################################################################################################
##Functions#################################################################################################################
############################################################################################################################

def parse_solver_options(options, output_level):
    solver_list = []
    solver_thresholds = []

    if options != None:
        for solver_args in options:
            if len(solver_args) < 2:
                print_solver_help_and_exit()

            solver_thresholds.append(float(solver_args[0]))
            if solver_args[1] == "neural":
                method = default_splot_method
                if len(solver_args) > 2:
                    method = int(solver_args[2])
                theta_files = default_theta_files
                if len(solver_args) > 3:
                    theta_files = solver_args[3].split(",")
                theta_file_lengths = []
                if len(solver_args) > 4:
                    theta_file_lengths = solver_args[4].split(",")
                    for i in range(len(theta_file_lengths)):
                        theta_file_lengths[i] = int(theta_file_lengths[i])
                while len(theta_file_lengths) < len(theta_files):
                    theta_file_lengths.append(-1)
                ymap_files = []
                if len(solver_args) > 5:
                    ymap_files = solver_args[5].split(",")
                while len(ymap_files) < len(theta_files):
                    ymap_files.append(default_ymap_file)
                solver_list.append(nnsolver.NnSolver(output_level, method, theta_file_lengths, theta_files, ymap_files))
            elif solver_args[1] == "phash":
                hashfile = default_hashfile
                if len(solver_args) > 2:
                    hashfile = solver_args[2]
                number_to_find = default_number_to_find
                if len(solver_args) > 3:
                    number_to_find = int(solver_args[3])
                solver_list.append(phashsolver.PhashSolver(output_level, hashfile, number_to_find))
            elif solver_args[1] == "md5":
                md5file = default_md5file
                if len(solver_args) > 2:
                    md5file = solver_args[2]
                wordmap_file = default_wordmap_file
                if len(solver_args) > 3:
                    wordmap_file = solver_args[3]
                solver_list.append(md5solver.Md5Solver(debug = output_level, md5_file =  md5file, word_map_file = wordmap_file))
            elif solver_args[1] == "human":
                entire_thing = True
                if len(solver_args) > 2:
                    entire_thing = False
                skip = 0.0
                if len(solver_args) > 3:
                    skip = float(skip)
                solver_list.append(humansolver.HumanSolver(debug = output_level, entire_thing = entire_thing, skip = skip))
            else:
                print "Uknown solver type '" + solver_args[1] + "'"
                print_solver_help_and_exit()
    return solver_list, solver_thresholds

def print_solver_help_and_exit():
    print "The -s options specifies a single solver in the chain.  Each solver can be specified multiple times.  Additional solvers will"
    print "be used as backup when the previous solver is unsure.  The available solver types are: neural, phash, md5, human"
    print "Each solver can have its own set of options.  The syntax to add a solver is as follows:\n"
    print "-s threshold solver_type [options]\n"
    print "Global options:"
    print "  threshold      = The percent/distance the solver needs be in order to not move on to the next solver.  For neural network"
    print "                     this value is the percentage certain the neural network is on a specific value.  Thus, a higher number"
    print "                     means the neural network is more certain.  For phash, this value is the sum of the distance from the"
    print "                     number_to_find closest matches.  Thus a smaller number means phash is more sure.  If no solvers can "
    print "                     determine a word with the specified amount certain, the last solver's choice is used."
    print "  solver_type    = The type of solver to use.  Available values: neural, phash, md5\n"
    print "Neural Network options:"
    print "  splot_bitmask  = A bitmask specifiying the methods used to convert the spectral plot from sox into the neural network\'s input"
    print "  theta_values   = A comma separated list of the theta matrix file to be used with by the neural network"
    print "  theta_value_lengths  = A comma separated list of the lengths of the inputs to use the corresponding theta file for"
    print "  ymap_file      = A comma separated list of the ymapping files for the corresponding theta files\n"
    print "Phash options:"
    print "  hashfile       = The file with all of the pHashes in it of other previously solved words"
    print "  number_to_find = The number of similar words to find when looking for similar spectrograms with find_cluster\n"
    print "Md5 options:"
    print "  md5sum file    = The file with all of the md5 sums and answers in it"
    print "  wordmap file   = The file with all of the mappings of index to word in it\n"
    print "Human options:"
    print "  'split'        = Tell the human solver to play each piece of the captcha instead of the whole thing\n"

    sys.exit()

